@@ -20,11 +20,11 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "common.h"
 #include "compat.h"
 #include "dict.h"
 #include "libyang.h"
 #include "log.h"
+#include "ly_common.h"
 #include "parser_data.h"
 #include "plugins_exts.h"
 #include "plugins_types.h"
@@ -775,7 +775,7 @@ schema_mount_dup_parent_ref(const struct lysc_ext_instance *ext, const struct ly
 
         /* get the referenced nodes (subtrees) */
         LYD_VALUE_GET(&term->value, xp_val);
-        if ((ret = lyd_find_xpath4(ctx_node, ctx_node, lyxp_get_expr(xp_val->exp), xp_val->format, xp_val->prefix_data,
+        if ((ret = lyd_find_xpath3(ctx_node, ctx_node, lyxp_get_expr(xp_val->exp), xp_val->format, xp_val->prefix_data,
                 NULL, &par_set))) {
             lyplg_ext_compile_log(NULL, ext, LY_LLERR, ret, "Parent reference \"%s\" evaluation failed.",
                     lyxp_get_expr(xp_val->exp));
@@ -903,8 +903,8 @@ schema_mount_validate(struct lysc_ext_instance *ext, struct lyd_node *sibling, c
         enum lyd_type data_type, uint32_t val_opts, struct lyd_node **diff)
 {
     LY_ERR ret = LY_SUCCESS;
-    uint32_t temp_lo = LY_LOSTORE_LAST, i;
-    struct ly_err_item *err;
+    uint32_t *prev_lo, temp_lo = LY_LOSTORE_LAST, i;
+    const struct ly_err_item *err;
     struct lyd_node *iter, *ext_data = NULL, *ref_first = NULL, *orig_parent = lyd_parent(sibling), *op_tree;
     struct lyd_node *ext_diff = NULL, *diff_parent = NULL;
     ly_bool ext_data_free = 0;
@@ -940,7 +940,7 @@ schema_mount_validate(struct lysc_ext_instance *ext, struct lyd_node *sibling, c
     }
 
     /* create accessible tree, remove LYD_EXT to not call this callback recursively */
-    lyd_unlink_siblings(sibling);
+    LY_CHECK_GOTO(lyd_unlink_siblings(sibling), cleanup);
     LY_LIST_FOR(sibling, iter) {
         iter->flags &= ~LYD_EXT;
     }
@@ -951,7 +951,7 @@ schema_mount_validate(struct lysc_ext_instance *ext, struct lyd_node *sibling, c
     }
 
     /* only store messages in the context, log as an extension */
-    ly_temp_log_options(&temp_lo);
+    prev_lo = ly_temp_log_options(&temp_lo);
 
     if (data_type == LYD_TYPE_DATA_YANG) {
         /* validate all the modules with data */
@@ -962,7 +962,7 @@ schema_mount_validate(struct lysc_ext_instance *ext, struct lyd_node *sibling, c
     }
 
     /* restore logging */
-    ly_temp_log_options(NULL);
+    ly_temp_log_options(prev_lo);
 
     /* restore sibling tree */
     for (i = 0; i < ref_set->count; ++i) {
@@ -982,7 +982,7 @@ schema_mount_validate(struct lysc_ext_instance *ext, struct lyd_node *sibling, c
         if (!err) {
             lyplg_ext_compile_log(NULL, ext, LY_LLERR, ret, "Unknown validation error (err code %d).", ret);
         } else {
-            lyplg_ext_compile_log_path(err->path, ext, LY_LLERR, err->no, "%s", err->msg);
+            lyplg_ext_compile_log_err(err, ext);
         }
         goto cleanup;
     }
